from Crypto.PublicKey import RSA
from base64 import b64encode
import unittest
import vcsecret

class TestVCSecret(unittest.TestCase):
    def setUp(self):
        key = RSA.generate(2048)
        self.private_key = b64encode(key.exportKey(format='DER'))
        self.public_key = b64encode(key.publickey().exportKey(format='DER'))

        self.private_secret = vcsecret.VCSecret(self.private_key)
        self.public_secret = vcsecret.VCSecret(self.public_key)

        self.secret = payload = self.public_secret.encrypt('secret')

    def test_encrypt(self):
        payload = self.public_secret.encrypt('test')

        self.assertEqual(4, len(payload.split(':')))
        self.assertTrue(payload.startswith('!!vcsecret:'))

    def test_decrypt(self):
        payload = self.private_secret.decrypt(self.secret)

        self.assertEqual('secret', payload)

    def test_decrypt_obj(self):
        dict_enc = {
                'a': 'not_a_secret',
                'b': self.secret,
                'c': [123, self.secret],
                'd': {'secret': self.secret, 'foo':b'bar'}
            }

        self.private_secret.decrypt_obj(dict_enc)
        self.assertEqual({
            'a': 'not_a_secret',
            'b': 'secret',
            'c': [123, 'secret'],
            'd': {'secret':'secret', 'foo':b'bar'}
            }, dict_enc)

    def test_decrypt_require_private_key(self):
        with self.assertRaises(vcsecret.CannotDecrypt):
            self.public_secret.decrypt(self.secret)




from base64 import b64encode, b64decode
from Crypto import Random
from Crypto.Cipher import PKCS1_OAEP, AES
from Crypto.PublicKey import RSA

# Because people are still using python 2 *boo*
def to_bytes(ints):
    return bytes(bytearray(ints))

def to_byte(c):
    return ord(c) if isinstance(c, str) else c

def _pkcs7_pad(payload):
    length = 16 - (len(payload) % 16)
    return payload + to_bytes([length]*length)

def _pkcs7_unpad(payload):
    return payload[:-to_byte(payload[-1])]

class CannotDecrypt(Exception):
    pass

class VCSecret:
    def __init__(self, key):
        self.key = RSA.importKey(b64decode(key))

    def encrypt(self, value):
        value_key = Random.get_random_bytes(16)
        value_iv = Random.get_random_bytes(16)
        value_cipher = AES.new(value_key, AES.MODE_CBC, value_iv)
        value_enc = value_cipher.encrypt(_pkcs7_pad(value.encode()))

        key_cipher = PKCS1_OAEP.new(self.key)
        key_enc = key_cipher.encrypt(value_key)

        return '!!vcsecret:{}:{}:{}'.format( \
                b64encode(key_enc).decode('utf-8'), \
                b64encode(value_iv).decode('utf-8'), \
                b64encode(value_enc).decode('utf-8'))

    def decrypt(self, payload):
        if not self.key.has_private():
            raise CannotDecrypt()

        (prefix, key_enc, value_iv, value_enc) = payload.split(':')
        if prefix != '!!vcsecret':
            raise InvalidSecretError('Secret "{}" should start with !!vcsecret, are you sure this secret is encrypted?'.format(payload))

        key_enc = b64decode(key_enc)
        value_iv = b64decode(value_iv)
        value_enc = b64decode(value_enc)

        key_cipher = PKCS1_OAEP.new(self.key)
        value_key = key_cipher.decrypt(key_enc)

        value_cipher = AES.new(value_key, AES.MODE_CBC, value_iv)
        value = _pkcs7_unpad(value_cipher.decrypt(value_enc))

        return value.decode('utf-8')

    def decrypt_obj(self, o):
        if isinstance(o, str) and o.startswith('!!vcsecret:'):
            return self.decrypt(o)
        elif isinstance(o, list):
            for i,v in enumerate(o):
                o[i] = self.decrypt_obj(v)
        elif isinstance(o, dict):
            for k,v in iter(o.items()):
                o[k] = self.decrypt_obj(v)
        return o




from vcsecret import VCSecret
from Crypto.PublicKey import RSA
from Crypto import Random
from base64 import b64encode
import argparse, sys

def main():
    parser = argparse.ArgumentParser()
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-g', '--generate', action='store_true')
    group.add_argument('-e', '--encrypt', action='store')
    group.add_argument('-s', '--generatesecret', action='store', type=int)
    parser.add_argument('-k', '--key', action='store', required=False)

    args = parser.parse_args()
    if args.generate:
        key = RSA.generate(2048)
        pk = b64encode(key.exportKey(format='DER')).decode('utf-8')
        pub = b64encode(key.publickey().exportKey(format='DER')).decode('utf-8')
        print('Public key:\n{}\n'.format(pub))
        print('Private key:\n{}'.format(pk))
    else:
        key = None
        if args.key:
            key = args.key
        else:
            try:
                key = open('.vckey', 'r').read()
            except FileNotFoundError:
                pass
        if not key:
            print('Require --key or .vskey')
            sys.exit(1)

        elif args.encrypt:
            secret = VCSecret(key)
            enc = secret.encrypt(args.encrypt)
            print(enc)
        elif args.generatesecret:
            secret = VCSecret(key)
            secret_value = b64encode(Random.get_random_bytes(args.generatesecret))
            print('Secret:\n{}\n'.format(secret_value))
            enc = secret.encrypt(secret_value)
            print('Encrypted secret:\n{}\n'.format(enc))
        else:
            parser.print_help()

if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

long_description = """
This Python utility and library is used to keep your secret value on Version Control Software. The secret value are encrypted with a public key so everyone can write them. However, the private key is required to decrypt those values. The private key should be stored only on the server running the app.
"""

setup(
    name='vcsecret',
    version='0.0.7',

    description='Keep your secrets in your VCS',
    long_description=long_description,
    url='https://github.com/isra17/vcsecret',
    author='isra17',
    author_email='isra017@gmail.com',
    license='LGPL3',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Security :: Cryptography',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities'
    ],

    keywords='secret config cryptography',

    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    entry_points={
        'console_scripts': [
            'vcsecret = vcsecret.main:main'
        ]
    },
    install_requires=['pycrypto']
)
